#!/bin/bash

MAKO=mako-render
PROFEAT=profeat
SCRIPTDIR=$(cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd)

usage()
{
cat <<-USAGE
usage: dist.sh [OPTIONS] TEMPLATE FROM TO

options:
    -n      only generate family instances for the largest family
    -v      show additional information
    -h      show this help text
USAGE
}

main()
{
    local opt
    while getopts ":hnv" opt; do
        case $opt in
            n)
                noredundant=1
                ;;
            v)
                verbose=1
                ;;
            h)
                usage
                exit
                ;;
            \?)
                echo "Invalid option: -$OPTARG" >&2
                ;;
        esac
    done
    shift $((OPTIND-1))

    if [[ $# != 3 ]]; then
        usage
        exit 1
    fi

    template=$1
    from=$2
    to=$3

    templatename=$(basename $template); templatename=${templatename%%.*}
    templatedir=$(dirname $template)

    make_outdir
    instantiate_families
    translate_all_in_one
    translate_one_by_one
    translate_properties

    if [[ -n "$noredundant" ]]; then touch $outdir/noredundant; fi
    echo $from > $outdir/from; echo $to > $outdir/to

    if [[ -f $templatedir/run-all.sh ]]; then
        cp $templatedir/run-all.sh $outdir
    fi
    cp $SCRIPTDIR/run.sh $outdir
    tar --directory=$templatedir/dist -czf ${templatename}.tar.gz $templatename
    log "Archive written to ${templatename}.tar.gz"
}

log()
{
    if [[ -n "$verbose" ]]; then echo $@; fi
}

make_outdir()
{
    outdir=$templatedir/dist/$templatename
    if [ -d $outdir ]; then
        read -p "$outdir already exists. Remove? [yY/nN] " -n 1 -r; echo
        if [[ $REPLY =~ ^[Yy]$ ]]; then rm -r $outdir; fi
    fi
    mkdir -p $outdir
}

instantiate_families()
{
    for i in $(seq $from $to); do
        log "Generating family instance of size $i"
        $MAKO $template --var=N=$i > $outdir/family_${i}.profeat
    done
}

translate_all_in_one()
{
    mkdir -p $outdir/all_in_one
    for i in $(seq $from $to); do
        log "Translating family instance of size $i to all-in-one model"
        translate $i all_in_one/${i}.prism
    done
}

translate_one_by_one()
{
    local from_local
    mkdir -p $outdir/one_by_one
    if [[ -n $noredundant ]]; then from_local=$to; else from_local=$from; fi
    for i in $(seq $from_local $to); do
        log "Translating family instance of size $i to one-by-one models"
        mkdir -p $outdir/one_by_one/$i
        translate $i one_by_one/${i}/instance.prism --one-by-one
    done
}

translate_properties()
{
    local props_base=$templatedir/${templatename}
    local file_props=${props_base}.props
    local file_fprops=${props_base}.fprops

    if [[ -f $file_props ]]; then
        cp $file_props $outdir/properties.props
    elif [[ -f $file_fprops ]]; then
        log "Translating properties"
        $PROFEAT $outdir/family_${to}.profeat $file_fprops -t -p $outdir/properties.props
    fi
}

translate()
{
    $PROFEAT $outdir/family_${1}.profeat -t -o $outdir/$2 $3
}

main "$@"

